#ifndef UU_UNPARSE_H
#define UU_UNPARSE_H

#include "ulib/UUID.h"

void uu_unparse_lower0(const struct_uu_t *in, char *out);
void uu_unparse_upper0(const struct_uu_t *in, char *out);
void uu_unparse0(const struct_uu_t *in, char *out);

void uu_unparse_lower1(const struct_uu_t *in, char *out);
void uu_unparse_upper1(const struct_uu_t *in, char *out);
void uu_unparse1(const struct_uu_t *in, char *out);

void uu_unparse_lower3(const struct_uu_t *in, char *out);
void uu_unparse_upper3(const struct_uu_t *in, char *out);
void uu_unparse3(const struct_uu_t *in, char *out);

void uu_unparse_lower4(const struct_uu_t *in, char *out);
void uu_unparse_upper4(const struct_uu_t *in, char *out);
void uu_unparse4(const struct_uu_t *in, char *out);

void uu_unparse_lower5(const struct_uu_t *in, char *out);
void uu_unparse_upper5(const struct_uu_t *in, char *out);
void uu_unparse5(const struct_uu_t *in, char *out);

void uu_unparse_lower6(const struct_uu_t *in, char *out);
void uu_unparse_upper6(const struct_uu_t *in, char *out);
void uu_unparse6(const struct_uu_t *in, char *out);

void uu_unparse_lower7(const struct_uu_t *in, char *out);
void uu_unparse_upper7(const struct_uu_t *in, char *out);
void uu_unparse7(const struct_uu_t *in, char *out);

#endif
/* ex:set ts=2 sw=2 itab=spaces: */
