use strict;
use warnings;
use Test::More;
use MyNote;
use UUID;

ok 1, 'loaded';

is UUID::_snap(), 0, 'snap init';

ok UUID::_snap(0.123), 'snap set';
is UUID::_snap(), 0.123, 'snap ok';

done_testing;
