use strict;
use warnings;
use Test::More;
use MyNote;
use Try::Tiny;

BEGIN {
    require UUID;
    ok 1, 'loaded';
    try {
        UUID->import(':snap=xxx');
    }
    catch {
        my $e = $_;
        # Non-numeric :snap argument at t/5snap/nonum.t line 11.
        #note $e;
        my $f = __FILE__;
        $f =~ s/\\/\\\\/g; # windows!
        like $e, qr/Non-numeric :snap argument at $f line/, 'correct';
    };
    ok 1, 'began';
}

ok 1, 'done';

done_testing;
